#include <stdio.h>
#include <string.h>
#include <stdlib.h>

char * change_path_prefix(const char * path, const char * prefix, const char * newprefix)
{
   /* something in prefix is required */
   if (strlen(prefix)<1)
     return NULL;

   /* ensure lengths are sane */
   if (strlen(path)<strlen(prefix))
     return NULL;

   /* ensure path contains and starts with prefix */
   if ( &path[0] != strstr(path, prefix))
     return NULL;
     
   /* ensure path starts with '/', so does prefix */
   if ( path[0] != '/' )
     return NULL;
   
   /* ensure prefix ends with one '/' or none */
   if (strlen(prefix)>=2 && prefix[strlen(prefix)-1] == '/' && prefix[strlen(prefix)-2] == '/')
     return NULL;

   int trailing_slash = (prefix[strlen(prefix)-1] == '/') ? 0 : 1;
   
   char *res;
   if (NULL == ( res = calloc(strlen(path) - strlen(prefix) + strlen(newprefix), sizeof(char))))
     return NULL;
   
   res = strncat(res, newprefix, strlen(newprefix) );
   res = strncat(res, &path[strlen(prefix)+trailing_slash], strlen(path) - strlen(prefix) + 1);

   return res;
}

#ifdef TESTCASE_CHANGE_PATH_PREFIX

int main(int argc, char *argv[])
{
   if (argc != 4) {
      printf("gimmi path prefix newprefix\n");
      exit(1);
   }
     
   printf("path: %s\n", argv[1]);
   printf("pref: %s\n", argv[2]);
   printf("newp: %s\n", argv[3]);

   char * changed = change_path_prefix(argv[1], argv[2], argv[3]);
   if (!changed) {
      printf("FAILED\n");
      exit(1);
   }

   printf("RESULT: %s\n", changed);

   free(changed);
   return 0;
}

#endif
